@@ -1,5 +1,6 @@
 import sys
 import re
+import importlib
 
 if __name__ == "__main__":
 
@@ -7,23 +8,13 @@ if __name__ == "__main__":
 
 
 from q2rad import Q2App, Q2Form
-from q2gui.q2dialogs import q2Mess, q2AskYN, q2WaitShow
+from q2gui.q2dialogs import q2Mess, q2AskYN, Q2WaitShow
 from q2gui.q2model import Q2CursorModel
 from q2db.schema import Q2DbSchema
 from q2db.db import Q2Db
 from q2rad.q2actions import Q2Actions
 from q2db.cursor import Q2Cursor
-from q2rad.q2raddb import (
-    read_url,
-    open_url,
-    insert,
-    raw_insert,
-    update,
-    delete,
-    transaction,
-    commit,
-    rollback,
-)
+from q2rad.q2raddb import *  # noqa:F403
 
 from q2gui import q2app
 from q2rad.q2raddb import q2cursor
@@ -36,6 +27,7 @@ from q2rad.q2modules import Q2Modules
 from q2rad.q2forms import Q2Forms
 from q2rad.q2lines import Q2Lines
 from q2rad.q2market import Q2Market
+from q2rad.q2packages import Q2Packages
 from q2rad.q2constants import Q2Constants, q2const
 from q2rad.q2queries import Q2Queries
 from q2rad.q2reports import Q2Reports, Q2RadReport
@@ -48,6 +40,7 @@ import os
 import json
 import subprocess
 import shutil
+import pkgutil
 
 
 q2_modules = ("q2rad", "q2gui", "q2db", "q2report")
@@ -68,6 +61,10 @@ def run_module(module_name):
     return q2app.q2_app.run_module(module_name)
 
 
+def run_form(form_name):
+    return q2app.q2_app.run_form(form_name)
+
+
 class Q2RadApp(Q2App):
     def __init__(self, title=""):
         super().__init__(title)
@@ -76,12 +73,13 @@ class Q2RadApp(Q2App):
 
         self.db_data = None
         self.db_logic = None
-        self.dev_mode = False
         self.selected_application = {}
         self.clear_app_info()
 
         self.q2market_path = "../q2market"
 
+        self.dev_mode = os.path.isdir(self.q2market_path)
+
         self.q2market_url = "https://raw.githubusercontent.com/AndreiPuchko/q2market/main/"
 
         self.assets_url = "https://raw.githubusercontent.com/AndreiPuchko/q2gui/main/assets/"
@@ -89,7 +87,7 @@ class Q2RadApp(Q2App):
         qss_file = "q2gui.qss"
         if not os.path.isfile(qss_file):
             qss_url = "https://raw.githubusercontent.com/AndreiPuchko/q2rad/main/q2gui.qss"
-            open(qss_file, "w").write(read_url(qss_url).decode("utf-8"))
+            open(qss_file, "w").write(read_url(qss_url).decode("utf-8"))  # noqa F405
 
         if os.path.isfile(qss_file):
             self.style_file = qss_file
@@ -126,7 +124,7 @@ class Q2RadApp(Q2App):
     def on_start(self):
         if not os.path.isfile("poetry.lock"):
             self.load_assets()
-            self.check_q2_update()
+            self.check_packages_update()
         self.open_application(autoload_enabled=True)
 
     def open_application(self, autoload_enabled=False):
@@ -134,6 +132,7 @@ class Q2RadApp(Q2App):
         if self.selected_application != {}:
             self.open_selected_app(True)
             self.check_app_update()
+            self.update_app_packages()
         else:
             self.close()
 
@@ -149,6 +148,7 @@ class Q2RadApp(Q2App):
         # self.run_modules()
         # self.run_reports()
         # self.run_app_manager()
+
         if go_to_q2market and (
             max(
                 [
@@ -194,6 +194,8 @@ class Q2RadApp(Q2App):
                 data_schema.add(**x)
 
         self.db_data.set_schema(data_schema)
+        if self.db_data.migrate_error_list:
+            q2Mess(self.db_data.migrate_error_list)
         self.create_menu()
 
     def migrate_db_logic(self):
@@ -205,11 +207,26 @@ class Q2RadApp(Q2App):
             Q2Queries(),
             Q2Actions(),
             Q2Reports(),
+            Q2Packages(),
         ):
             for x in form.get_table_schema():
                 data_schema.add(**x)
         self.db_logic.set_schema(data_schema)
 
+    def get_autocompletition_list(self):
+        rez = []
+        tables = self.db_data.db_schema.get_schema_tables()
+        for ta in tables:
+            rez.append(ta)
+            rez.append(f"d.{ta}")
+            for co in tables[ta]["columns"]:
+                rez.append(f"d.{ta}.{co}")
+                rez.append(f"{ta}.{co}")
+        for x in q2cursor("select const_name from  constants").records():
+            rez.append("c.const.{const_name}".format(**x))
+            rez.append("const.{const_name}".format(**x))
+        return rez
+
     def open_databases(self):
         self.db_data = Q2Db(database_name=self.selected_application["database_data"])
         self.db_logic = Q2Db(database_name=self.selected_application["database_logic"])
@@ -229,12 +246,14 @@ class Q2RadApp(Q2App):
         self.dev_mode = self.selected_application.get("dev_mode") or os.path.isdir(self.q2market_path)
 
         if self.dev_mode:
-            self.add_menu("Dev|Forms", self.run_forms, toolbar=self.dev_mode and 0)
-            self.add_menu("Dev|Modules", self.run_modules, toolbar=self.dev_mode and 0)
-            self.add_menu("Dev|Querys", self.run_queries, toolbar=self.dev_mode and 0)
-            self.add_menu("Dev|Reports", self.run_reports, toolbar=self.dev_mode and 0)
+            self.add_menu("Dev|Forms", self.run_forms, toolbar=self.dev_mode)
+            self.add_menu("Dev|Modules", self.run_modules, toolbar=self.dev_mode)
+            self.add_menu("Dev|Querys", self.run_queries, toolbar=self.dev_mode)
+            self.add_menu("Dev|Reports", self.run_reports, toolbar=self.dev_mode)
+            self.add_menu("Dev|Packages", self.run_packages, toolbar=self.dev_mode)
         self.build_menu()
         # self.show_toolbar(False)
+        pass
 
     def about(self, text=""):
         about = []
@@ -252,7 +271,7 @@ class Q2RadApp(Q2App):
         about.append("<b>q2RAD</b>")
         about.append("Versions:")
         about.append(f"<b>Python</b>: {sys.version}<p>")
-        w = q2WaitShow(len(q2_modules))
+        w = Q2WaitShow(len(q2_modules))
         for package in q2_modules:
             w.step()
             latest_version, current_version = self.get_package_versions(package)
@@ -271,7 +290,7 @@ class Q2RadApp(Q2App):
     def asset_file_loader(self, name):
         asset_url = f"{self.assets_url}/{name}"
         try:
-            asset_content = read_url(asset_url)
+            asset_content = read_url(asset_url)  # noqa F405
             if asset_content is None:
                 return False
         except Exception:
@@ -303,7 +322,7 @@ class Q2RadApp(Q2App):
         icons = [getattr(q2app, x) for x in dir(q2app) if x.endswith("ICON")]
         icons.append("q2gui.ico")
 
-        w = q2WaitShow(len(icons))
+        w = Q2WaitShow(len(icons))
         errors = []
         for x in icons:
             w.step(x)
@@ -363,41 +382,27 @@ class Q2RadApp(Q2App):
             self.make_desktop_shortcut()
 
     def get_package_versions(self, package):
-        response = open_url(f"https://pypi.python.org/pypi/{package}/json")
+        response = open_url(f"https://pypi.python.org/pypi/{package}/json")  # noqa F405
         if response:
             latest_version = json.load(response)["info"]["version"]
         else:
             latest_version = None
-        current_version = sys.modules[package].__version__
+        current_version = sys.modules[package].__version__ if package in sys.modules else ""
         return latest_version, current_version
 
-    def update_packages(self):
+    def update_packages(self, packages_list):
         upgraded = []
-        w = q2WaitShow(len(q2_modules))
-        for package in q2_modules:
+        w = Q2WaitShow(len(packages_list))
+        for package in packages_list:
             if w.step(package):
                 break
             latest_version, current_version = self.get_package_versions(package)
             if latest_version != current_version and latest_version:
-
-                runpip = lambda: subprocess.check_call(  # noqa:E731
-                    [
-                        sys.executable.replace("w.exe", ".exe"),
-                        "-m",
-                        "pip",
-                        "install",
-                        "--upgrade",
-                        "--no-cache-dir",
-                        f"{package}=={latest_version}",
-                    ],
-                    shell=True if "win" in sys.platform else False,
-                )
-
                 try:
-                    runpip()
+                    self.pip_install(package, latest_version)
                 except Exception:
                     try:
-                        runpip()
+                        self.pip_install(package, latest_version)
                     except Exception:
                         pass
 
@@ -418,10 +423,38 @@ class Q2RadApp(Q2App):
             # self.close()
         pass
 
+    def pip_install(self, package, latest_version):
+        return subprocess.check_call(
+            [
+                sys.executable.replace("w.exe", ".exe"),
+                "-m",
+                "pip",
+                "install",
+                "--upgrade",
+                "--no-cache-dir",
+                f"{package}=={latest_version}",
+            ],
+            shell=True if "win" in sys.platform else False,
+        )
+
+    def pip_uninstall(self, package):
+        return subprocess.check_call(
+            [
+                sys.executable.replace("w.exe", ".exe"),
+                "-m",
+                "pip",
+                "uninstall",
+                "-y",
+                f"{package}",
+            ],
+            shell=True if "win" in sys.platform else False,
+        )
+
     def check_app_update(self):
-        if self.app_url and self.app_version:
-            market_version = read_url(self.app_url + ".version").decode("utf-8")
-            print(self.app_url + ".version")
+        # self.update_app_packages()
+
+        if not os.path.isdir(self.q2market_path) and self.app_url and self.app_version:
+            market_version = read_url(self.app_url + ".version").decode("utf-8")  # noqa F405
             if market_version != self.app_version:
                 if (
                     q2AskYN(
@@ -432,27 +465,47 @@ class Q2RadApp(Q2App):
                     )
                     == 2
                 ):
-                    data = json.load(open_url(self.app_url + ".json"))
+                    data = json.load(open_url(self.app_url + ".json"))  # noqa F405
                     AppManager.import_json_app(data)
                     self.open_selected_app()
 
-    def check_q2_update(self):
+    def update_app_packages(self):
+        # if os.path.isdir(".vscode"):
+        #     # because of falls when running under VSCODE
+        #     return
+
+        extra_packages = [
+            x["package_name"] for x in q2cursor("select * from packages", self.db_logic).records()
+        ]
+        # installed_packages = [x.name for x in pkgutil.iter_modules()]
+
+        # for x in extra_packages:
+        #     if x in installed_packages:
+        #         __import__(x)
+        self.check_packages_update(extra_packages)
+
+    def check_packages_update(self, packages_list=q2_modules):
         can_upgrade = False
-        for package in q2_modules:
+        list_2_upgrade = []
+        for package in packages_list:
             latest_version, current_version = self.get_package_versions(package)
-            can_upgrade = latest_version != current_version
-            if can_upgrade:
-                break
+            if latest_version != current_version and current_version:
+                list_2_upgrade.append(f"<b>{package}</b>: {current_version} > {latest_version}")
+                if not can_upgrade:
+                    can_upgrade = True
+            # if can_upgrade:
+            #     break
         if can_upgrade:
             if (
                 q2AskYN(
-                    "Updates for q2* packages are avaiable!<p><p>"
+                    "Updates for packages are avaiable!<p><p>"
+                    f"{',<br> '.join(list_2_upgrade)}<br><br>"
                     "Do you want to proceed with update?<p><p>"
                     "The program will be restarted after the update!"
                 )
                 == 2
             ):
-                self.update_packages()
+                self.update_packages(packages_list)
 
     def run_constants(self):
         Q2Constants().run()
@@ -505,6 +558,9 @@ class Q2RadApp(Q2App):
     def run_reports(self):
         Q2Reports().run()
 
+    def run_packages(self):
+        Q2Packages().run()
+
     def run_form(self, name):
         self.get_form(name).run()
 
@@ -513,6 +569,9 @@ class Q2RadApp(Q2App):
             return
         form_dic = self.db_logic.get("forms", f"name ='{name}'")
 
+        if form_dic == {}:
+            return None
+
         sql = f"""
             select
                 column
@@ -540,7 +599,7 @@ class Q2RadApp(Q2App):
             where name = '{name}'
             order by seq
             """
-        cu = q2cursor(sql, self.db_logic)
+        cu: Q2Cursor = q2cursor(sql, self.db_logic)
 
         form = Q2Form(form_dic["title"])
         form.no_view_action = 1
@@ -577,6 +636,7 @@ class Q2RadApp(Q2App):
         if form_dic["form_table"]:
             form_cursor: Q2Cursor = self.db_data.table(table_name=form_dic["form_table"])
             form_model = Q2CursorModel(form_cursor)
+            form_model.set_order(form_dic["form_table_sort"]).refresh()
             form.set_model(form_model)
             sql = f"select * from actions where name = '{name}' order by seq"
             cu = q2cursor(sql, self.db_logic)
@@ -588,12 +648,21 @@ class Q2RadApp(Q2App):
                 else:
                     if x["child_form"] and x["child_where"]:
                         child_form_name = x["child_form"]
+
+                        def get_action_form(child_form_name):
+                            def worker():
+                                return self.get_form(child_form_name)
+
+                            return worker
+
                         form.add_action(
                             x["action_text"],
                             self.code_runner(x["action_worker"]) if x["action_worker"] else None,
-                            child_form=lambda: self.get_form(child_form_name),
+                            child_form=get_action_form(child_form_name),
                             child_where=x["child_where"],
                             hotkey=x["action_key"],
+                            child_noshow=x["child_noshow"],
+                            child_copy_mode=x["child_copy_mode"],
                             eof_disabled=1,
                         )
                     else:
@@ -601,8 +670,11 @@ class Q2RadApp(Q2App):
                             x["action_text"],
                             self.code_runner(x["action_worker"], form=form) if x["action_worker"] else None,
                             hotkey=x["action_key"],
+                            child_noshow=x["child_noshow"],
+                            child_copy_mode=x["child_copy_mode"],
                             eof_disabled=x["eof_disabled"],
                         )
+
         return form
 
     def code_error(self):
@@ -668,6 +740,7 @@ class Q2RadApp(Q2App):
                 "__name__": __name__,
             }
             code = self.code_compiler(script)
+            # print("code compiled", "\n" ,script)
             if code["code"]:
                 try:
                     exec(code["code"], globals(), __locals_dict)
